/*
 * Copyright 2010 Trustees of the University of Pennsylvania Licensed under the
 * Educational Community License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License. You may
 * obtain a copy of the License at
 *
 * http://www.osedu.org/licenses/ECL-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS"
 * BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */
package metridoc.sushi.client.web;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.http.HttpResponse;
import org.apache.http.client.ResponseHandler;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import metridoc.sushi.client.SushiClient;
import metridoc.sushi.client.SushiResponseParser;

/**
 * Servlet implementation class SushiServlet
 * 
 * @author Narine Ghochikyan
 */
public class SushiServlet extends HttpServlet {
	private static final long serialVersionUID = 1L;

	private static final Logger LOG = LoggerFactory.getLogger(SushiServlet.class);

	/**
	 * @see HttpServlet#doGet(HttpServletRequest request, HttpServletResponse
	 *      response)
	 */
	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		response.sendRedirect("");
	}

	/**
	 * @see HttpServlet#doPost(HttpServletRequest request, HttpServletResponse
	 *      response)
	 */
	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException,
			IOException {
		SushiResponseHandler responseHandler = new SushiResponseHandler(response, "sushi_"
				+ request.getParameter("requestorId"));
		SushiClient.runRequest(getSushiParameterMap(request), responseHandler);
	}

	private Map<String, Object> getSushiParameterMap(HttpServletRequest request) {
		Map<String, Object> map = new HashMap<String, Object>();
		map.put("monthsCount", new Integer(request.getParameter("monthsCount")));
		map.put("sushiUrl", request.getParameter("sushiUrl"));
		map.put("requestorId", request.getParameter("requestorId"));
		map.put("requestorName", request.getParameter("requestorName"));
		map.put("requestorEmail", request.getParameter("requestorEmail"));
		map.put("customerId", request.getParameter("customerId"));
		map.put("customerName", request.getParameter("customerName"));
		//map.put("reportRequestId", request.getParameter("reportRequestId"));
		map.put("reportName", request.getParameter("reportName"));
		map.put("reportRelease", request.getParameter("reportRelease"));
		return map;
	}

	static class SushiResponseHandler implements ResponseHandler<Boolean> {
		private HttpServletResponse finalResponse;
		private String responseFileName;

		public SushiResponseHandler(HttpServletResponse finalResponse, String responseFileName) {
			this.finalResponse = finalResponse;
			this.responseFileName = responseFileName;
		}

		public Boolean handleResponse(HttpResponse response) {
			try {
				int responseStatus = response.getStatusLine().getStatusCode();
				if (responseStatus == HttpServletResponse.SC_OK) {
					finalResponse.setHeader("Content-Disposition", "attachment;filename=\"" + responseFileName
							+ ".csv\"");
					finalResponse.setContentType("text/csv");
					
					SushiResponseParser responseParser = new SushiResponseParser();
					responseParser.setOutputStream(finalResponse.getOutputStream());
					responseParser.process(response.getEntity().getContent());
					return true;
				}
				sendInternalError(finalResponse,
						"The request to specified sushi URL failed: '" + responseStatus + "-"
								+ response.getStatusLine().getReasonPhrase() + "'.");
			} catch (IOException e) {
				LOG.error("IO exceprion occured while preparing Final Response", e);
				sendInternalError(finalResponse, "An unexpected error occurred while processing the request.");
			}catch (Exception ex){
				LOG.error("Exceprion occured while preparing Final Response", ex);
				sendInternalError(finalResponse, ex.getMessage());
			}
			return false;
		}
		
		private static void sendInternalError(HttpServletResponse response, String message){
			response.reset();
			try {
				response.sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR,
						message);
			} catch (Exception e1) {
				LOG.error("Failed to send error message", e1);
			}
		}
	}
	
}
